import { createElement, FunctionComponent } from 'react'
import loadable from 'react-loadable'
import { load, loading } from './channel'
import { PluggableRoute } from './index'

/**
 * @type {Function} loading函数组件
 */
let Loading = () => null
/**
 * 使用自定义Loading组件，将会在动态加载module时，为用户展示Loading UI
 *
 * 这个Loading组件将会被loadable所用，组件的入参可以参考[LoadingComponent](https://github.com/jamiebuilds/react-loadable#loadingcomponent)
 * @param {Function} fn
 */
export function useCustomLoading (fn) {
  Loading = fn
}

const loadedComponent = {}
enum LoadStatus {
  loading = 'loading',
  loaded = 'loaded'
}
/**
 * 标记loadedComponent的loading和loaded状态
 */
function markLoadStatus (key: string, cb: () => void, status: LoadStatus) {
  switch (status) {
    case 'loading':
      if (!loadedComponent.hasOwnProperty(key)) {
        cb()
        loadedComponent[key] = undefined // 第一次标记loading状态
      }
      break
    case 'loaded':
      if ((key in loadedComponent) && loadedComponent[key] !== true) {
        cb()
        loadedComponent[key] = true // 第二次标记已经加载状态
      }
      break
  }
}
export function wrapWithLoadable (route: PluggableRoute) {
  return loadable({
    // delay: 1000,
    loader: route.content || route.Content,
    loading (props) {
      markLoadStatus(route.getNest(), () => {
        // console.debug('loading route component', route)
        loading(route)
      }, LoadStatus.loading)
      return createElement(Loading, props)
    },
    render ({ default: LoadableComponent }: { default: FunctionComponent }, props) {
      markLoadStatus(route.getNest(), () => {
        // console.debug('loaded route component', route)
        load(route)
      }, LoadStatus.loaded)
      return createElement(LoadableComponent, props)
    }
  })
}
